﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using System.IO;

namespace XmlTree
{
    public partial class XmlForm : Form
    {
        MyData data = new MyData();

        public XmlForm()
        {
            InitializeComponent();
            data.Text = "abc";
            data.Value = 7;
            propertyGrid.SelectedObject = data;
            tabControl.SelectedTab = propertyPage;
        }

        private void saveDataButton_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                // using System.IO;
                // using System.Xml.Serialization;
                StreamWriter f = new StreamWriter(saveFileDialog.FileName);
                XmlSerializer s = new XmlSerializer(data.GetType());
                s.Serialize(f, data);
                f.Close();
            }
        }

        private void showInfo(string s)
        {
            statusLabel.Text = s;
            logBox.AppendText (s + "\r\n");
        }

        private void showObj (object p)
        {
            propertyGrid.SelectedObject = p;
        }

        TreeNode current = null;

        private void runButton_Click(object sender, EventArgs e)
        {
            // readSimpleXml("..\\..\\data.xml");
            // readDom ("..\\..\\data.xml");

        }

        private void nextButton_Click(object sender, EventArgs e)
        {
        }

        protected void showDom (XmlNode node, TreeNode top)
        {
            MyTreeNode n = new MyTreeNode ();
            n.obj = node;
            if (node.NodeType == XmlNodeType.Text)
                n.Text = node.Value;
            else if (node.NodeType == XmlNodeType.Element)
                n.Text = "<" + node.Name + ">" ;
            else
                n.Text = node.Name;

            if (top == null)
                treeView.Nodes.Add (n);
            else
                top.Nodes.Add(n);

            if (node.Attributes != null)
            foreach (XmlAttribute attr in node.Attributes)
            {
                MyTreeNode a = new MyTreeNode();
                a.obj = attr;
                a.Text = attr.Name + " = " + attr.Value;
                n.Nodes.Add(a);
            }

            foreach (XmlNode child in node.ChildNodes)
                showDom (child, n);
        }

        private XmlDocument doc = null;

        protected void readDom(string fileName)
        {
            doc = new XmlDocument();
            doc.Load (fileName);
            showObj (doc.DocumentElement);
            showDom ( doc.DocumentElement, null);
        }

        private void saveButton_Click(object sender, EventArgs e)
        {
            if (doc != null)
                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                    doc.Save (saveFileDialog.FileName);
        }

        private void treeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            TreeNode n = treeView.SelectedNode;
            MyTreeNode m = n as MyTreeNode;
            if (m != null)
                showObj (m.obj);
        }

        // using System.Xml;
        XmlTextReader f;

        protected void readSimpleXml(string fileName)
        {
            f = new XmlTextReader("..\\..\\data.xml");
            while (f.Read())
            {
                showObj(f);
                if (f.NodeType == XmlNodeType.Element)
                {
                    TreeNode n = new TreeNode();
                    n.Text = "<" + f.Name + ">";
                    showInfo("Element: " + f.Name);

                    if (current == null)
                        treeView.Nodes.Add(n);
                    else
                        current.Nodes.Add(n);

                    current = n;

                    if (f.HasAttributes)
                    {
                        for (int i = 0; i < f.AttributeCount; i++)
                        {
                            f.MoveToAttribute(i);
                            TreeNode a = new TreeNode();
                            a.Text = f.Name + " = " + f.Value;
                            current.Nodes.Add(a);
                        }
                        f.MoveToElement();
                    }

                    if (f.IsEmptyElement)
                        current = current.Parent;
                }
                else if (f.NodeType == XmlNodeType.EndElement)
                {
                    current = current.Parent;
                }
                else if (f.NodeType == XmlNodeType.Text)
                {
                    TreeNode n = new TreeNode();
                    n.Text = f.Value;
                    current.Nodes.Add(n);
                }
            }
        }


    }
}
